use crate::{CdnInfo, Experiment, LanguageMap};
use serde::{Deserialize, Serialize};
use std::collections::HashMap;

#[derive(Serialize, Deserialize, Debug)]
#[serde(untagged)]
pub enum LauncherConfig {
    LauncherConfigV1(LauncherConfigV1),
    LauncherConfigV2(LauncherConfigV2),
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct LauncherConfigV1 {
    pub default: LauncherDefaultConfigV1,
    pub crash_init_switch: u32,
    pub animate_bg_switch: u32,
    pub animate_background: Option<AnimateBackground>,
    pub animate_background_language: Option<HashMap<String, AnimateBackground>>,
    pub navigation_bar_switch: Option<u32>,
    pub navigation_bar_language: Option<LanguageMap>,
    pub resources_gray: Option<ResourcesGray>,
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct LauncherDefaultConfigCommon {
    pub cdn_list: Vec<CdnInfo>,
    pub changelog: LanguageMap,
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct LauncherDefaultConfigV1 {
    #[serde(flatten)]
    pub common: LauncherDefaultConfigCommon,
    pub installer: String,
    #[serde(rename = "installerMD5")]
    pub installer_md5: String,
    pub installer_size: u64,
    pub version: String,
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct AnimateBackground {
    pub url: String,
    pub md5: String,
    pub frame_rate: u32,
    pub duration_in_second: u32,
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct LauncherConfigV2 {
    pub default: LauncherDefaultConfigV2,
    pub crash_init_switch: u32,
    pub navigation_bar_switch: Option<u32>,
    pub navigation_bar_language: Option<LanguageMap>,
    pub resources_gray: Option<ResourcesGray>,
    pub function_code: Option<FunctionCode>,
    pub experiment: Option<Experiment>,
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct LauncherDefaultConfigV2 {
    #[serde(flatten)]
    pub common: LauncherDefaultConfigCommon,
    pub resource: LauncherDefaultConfigResource,
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct LauncherDefaultConfigResource {
    pub file_check_list_md5: String,
    pub md5: String,
    pub path: String,
    pub size: u64,
    pub version: String,
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct ResourcesGray {
    pub gray_url: Option<String>,
    pub gray_switch: u32,
}

#[derive(Serialize, Deserialize, Debug)]
#[serde(rename_all = "camelCase")]
pub struct FunctionCode {}
